﻿namespace TabsStudioTrackEd
{
    class Controller
    {
        public Controller(TabsStudioExt.ITabsStudioEngine engine_)
        {
            engine = engine_;
            docs = new System.Collections.Generic.Dictionary<string, DocStatus>(System.StringComparer.OrdinalIgnoreCase);
        }

        public void OnDocDataChanged(string file, Microsoft.VisualStudio.Shell.Interop.__VSRDTATTRIB a)
        {
            DocStatus status = DocStatus.Fresh;
            switch (a)
            {
                case Microsoft.VisualStudio.Shell.Interop.__VSRDTATTRIB.RDTA_DocDataReloaded:
                    status = DocStatus.Fresh;
                    break;
                case Microsoft.VisualStudio.Shell.Interop.__VSRDTATTRIB.RDTA_DocDataIsDirty:
                    status = DocStatus.Dirty;
                    break;
                case Microsoft.VisualStudio.Shell.Interop.__VSRDTATTRIB.RDTA_DocDataIsNotDirty:
                    if (docs.ContainsKey(file) && docs[file] == DocStatus.Dirty)
                        status = DocStatus.Saved;
                    break;
                //default:
                // FAIL();
            }
            docs[file] = status;
            UpdateFileMarker(file, status);
        }

        public void OnTabExtensionCreated(TabsStudioExt.TabExtensionEventArgs e)
        {
            if (e.Tab.TabExtensions.Count == 2)
            {
                UpdateTabNameMarker(e.Tab, DocStatus.Fresh);
                UpdateTabExtensionMarker(e.Tab.TabExtensions[0]);
                UpdateTabExtensionMarker(e.Tab.TabExtensions[1]);
            }
        }

        public void OnTabExtensionDestroyed(TabsStudioExt.TabExtensionEventArgs e)
        {
            if (e.Tab.TabExtensions.Count == 2)
            {
                TabsStudioExt.TabExtension remainingExtension = e.Tab.TabExtensions[0].Window == e.TabExtension.Window ?
                        e.Tab.TabExtensions[1] : e.Tab.TabExtensions[0];
                UpdateTabNameMarker(e.Tab, CachedDocStatus(GetDocumentFullName(remainingExtension.Window)));
            }
        }

        private void UpdateFileMarker(string file, DocStatus s)
        {
            foreach (TabsStudioExt.Tabs tabs in engine.ReturnTabs())
            {
                foreach (TabsStudioExt.Tab tab in tabs.TabList)
                {
                    if (tab.TabExtensions.Count == 1)
                    {
                        if (IsMatch(tab.TabExtensions[0], file))
                            UpdateTabNameMarker(tab, s);
                    }
                    if (tab.TabExtensions.Count > 1)
                    {
                        foreach (TabsStudioExt.TabExtension ext in tab.TabExtensions)
                        {
                            if (IsMatch(ext, file))
                                UpdateTabExtensionMarker(ext, s);
                        }
                    }
                }
            }
        }

        private bool IsMatch(TabsStudioExt.TabExtension ext, string file)
        {
            return System.StringComparer.OrdinalIgnoreCase.Compare(file, GetDocumentFullName(ext.Window)) == 0;
        }

        private string GetDocumentFullName(EnvDTE.Window w)
        {
            if (w.Type == EnvDTE.vsWindowType.vsWindowTypeDocument)
            {
                try
                {
                    return w.Document.FullName;
                }
                catch (System.Exception)
                {
                }
            }
            return null;
        }

        private void UpdateTabNameMarker(TabsStudioExt.Tab tab, DocStatus s)
        {
            System.Windows.Controls.DockPanel tabInternals = tab.TabItem.Header as System.Windows.Controls.DockPanel;
            if (tabInternals == null || tabInternals.Children.Count < 1)
                return;

            System.Windows.Controls.StackPanel nameGroup = tabInternals.Children[0] as System.Windows.Controls.StackPanel;
            Properties.SetDocStatus(nameGroup, s);
        }

        private void UpdateTabExtensionMarker(TabsStudioExt.TabExtension ext, DocStatus s)
        {
            System.Windows.Controls.StackPanel tabExtensionGroup = ext.Label.Parent as System.Windows.Controls.StackPanel;
            Properties.SetDocStatus(tabExtensionGroup, s);
        }

        private void UpdateTabExtensionMarker(TabsStudioExt.TabExtension ext)
        {
            UpdateTabExtensionMarker(ext, CachedDocStatus(GetDocumentFullName(ext.Window)));
        }

        private DocStatus CachedDocStatus(string file)
        {
            return (file != null && docs.ContainsKey(file)) ? docs[file] : DocStatus.Fresh; 
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private System.Collections.Generic.Dictionary<string, DocStatus> docs;
    }
}
