﻿namespace TabsStudioTrackEd
{
    public class TrackEd : TabsStudioExt.ITabsStudioAddin, TabsStudioExt.IStyler
    {
        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte)
        {
            engine = engine_;
            controller = new Controller(engine);

            engine.Initialize += OnInitialize;
            engine.TabExtensionCreated += OnTabExtensionCreated;
            engine.TabExtensionDestroyed += OnTabExtensionDestroyed;

            System.IServiceProvider serviceProvider = new Microsoft.VisualStudio.Shell.ServiceProvider(
                    dte as Microsoft.VisualStudio.OLE.Interop.IServiceProvider);
            rdt = (Microsoft.VisualStudio.Shell.Interop.IVsRunningDocumentTable)
                serviceProvider.GetService(typeof(Microsoft.VisualStudio.Shell.Interop.SVsRunningDocumentTable));
            docEventsSink = new RunningDocTableEventsSink(rdt);
            docEventsSink.DocDataChanged += OnDocDataChanged;
            rdt.AdviseRunningDocTableEvents(docEventsSink, out sinkCookie);
        }

        public void OnDisconnection()
        {
            engine.Initialize -= OnInitialize;
            engine.TabExtensionCreated -= OnTabExtensionCreated;
            engine.TabExtensionDestroyed -= OnTabExtensionDestroyed;
            
            rdt.UnadviseRunningDocTableEvents(sinkCookie);
            docEventsSink.DocDataChanged -= OnDocDataChanged;
        }

        public string GetNamespacesForResourceDictionary()
        {
            return "xmlns:TabsStudioTrackEd=\"clr-namespace:TabsStudioTrackEd;assembly=TrackEd\"";
        }

        private void OnInitialize(object sender, System.EventArgs e)
        {
            UpdatePresentationStyle();
        }

        private void OnDocDataChanged(string file, Microsoft.VisualStudio.Shell.Interop.__VSRDTATTRIB a)
        {
            controller.OnDocDataChanged(file, a);
        }

        private void OnTabExtensionCreated(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            controller.OnTabExtensionCreated(e);
        }

        private void OnTabExtensionDestroyed(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            controller.OnTabExtensionDestroyed(e);
        }

        private void UpdatePresentationStyle()
        {
            TabsStudioExt.Presentation presentation = new TabsStudioExt.Presentation("", "");
            {
                TabsStudioExt.PresentationStyle style = new TabsStudioExt.PresentationStyle();
                style.Setters.Add(LoadString("TabNameModificationMarkerTemplate.xml"));
                presentation.Styles.Add("TabsStudio:TabNameModificationMarker", style);
            }
            {
                TabsStudioExt.PresentationStyle style = new TabsStudioExt.PresentationStyle();
                style.Setters.Add(LoadString("TabExtensionModificationMarkerTemplate.xml"));
                presentation.Styles.Add("TabsStudio:TabExtensionModificationMarker", style);
            }
            engine.UpdatePresentationStyles("TrackEd", presentation);
        }

        private static string LoadString(string resource)
        {
            using (System.IO.Stream stream = System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(
                "TrackEd." + resource))
            {
                using (System.IO.StreamReader reader = new System.IO.StreamReader(stream))
                {
                    return reader.ReadToEnd();
                }
            }
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private Controller controller;
        private Microsoft.VisualStudio.Shell.Interop.IVsRunningDocumentTable rdt;
        RunningDocTableEventsSink docEventsSink;
        private uint sinkCookie;
    }
}
